UNIT Graphics; (* Version 2.0 *)

INTERFACE

 USES Crt, Dos;

 PROCEDURE GrafikModus;
 PROCEDURE Loeschen (Color: Byte);
 PROCEDURE TextModus;
 PROCEDURE Pixel (x, y: WORD; Color: BYTE);
 FUNCTION  PixelFarbe (x, y: WORD): BYTE;
 PROCEDURE Schreiben (x, y: WORD; Text: STRING; 
                      Vordergrund, Hintergrund: BYTE);
 PROCEDURE Linie (x0, y0, x1, y1, Farbe: WORD);
 FUNCTION  Taste: CHAR;

IMPLEMENTATION

 PROCEDURE GrafikModus;

  CONST
   EGA_Regs    : ARRAY [0..16] OF BYTE  = (0,1,2,3,4,5,6,7,8,9,10,11,
                                           12,13,14,15,0);
  TYPE
   PalTyp      = RECORD
                  r, g, b   : BYTE
                 END;
  VAR
   Regs: Registers;

  BEGIN
   Regs.ax := $0012; (* GrafikModus $12 640x480x16 *)
   Intr ($10, Regs);
 END; (* Grafikmodus *)

 PROCEDURE Loeschen(Color: Byte);
  (* -------------------------------------------------- *)
  (* Lscht den gesamten Bildschirm mit der Farbe Color *)
  (* Verbesserte Set/Reset-Version                      *)
  (* -------------------------------------------------- *)
   CONST
   ScrSeg = $A000;   (* Segmentadresse des EGA/VGA-Bildspeichers *)
   VAR
    PageSize: Word ABSOLUTE $0000:$044C; (* BIOS: Bildseitengre *)
  BEGIN
   Port[$3CE] := 0;               (* Index fr Set/Reset Register *)
   Port[$3CF] := Color;           (* Farbnummer laden *)
   Port[$3CE] := 1;               (* Index fr Enable Set/Reset Reg. *)
   Port[$3CF] := $0F;             (* Set/Reset als Datenquelle *)
   FillChar(Mem[ScrSeg:0],PageSize,0); (* Schreibvorgang auslsen *)
   Port[$3CF] := 0;               (* Default: CPU als Datenquelle *)
 END;


 PROCEDURE Pixel (x,y: Word; Color: Byte);
  CONST
   ScrSeg = $A000;   (* Segmentadresse des EGA/VGA-Bildspeichers *)
  VAR
    ScrWidth: Word ABSOLUTE $0:$044A;  (* BIOS: Spalten/Bytes pro Zeile *)
    ScrOfs:   Word;
    BitMask:  Byte;

   BEGIN
    IF (x > 639) OR (x < 0) OR (y > 479) OR (y < 0) THEN EXIT;
    Inline ($FA);
    y := 479 - y;
    ScrOfs  := y * ScrWidth + x DIV 8;  (* Bildspeicher-Offsetadresse *)
    BitMask := $80 SHR (x AND 7);       (* Bitmaske berechnen *)
    PortW[$3CE] := Color SHL 8;         (* Index fr Set/Reset Register *)
    PortW[$3CE] := $0F01;               (* Index fr Enable Set/Reset Reg. *)
    PortW[$3CE] := 8 + BitMask SHL 8;   (* Index fr Bit Mask Register *)
    Inc (Mem[ScrSeg:ScrOfs]);           (* Lese/Schreibzyklus erzeugen! *)
    Inline ($FB);
   END; (* Plot *)

  PROCEDURE TextModus;
   BEGIN
    TEXTMODE (Co80);
   END;

  PROCEDURE Linie (x0, y0, x1, y1, Farbe: WORD);
    VAR
      dx, dy, x, y: INTEGER;
      m: REAL;
    BEGIN
      (* senkrechte Linie *)
      IF x0 = x1 THEN
        BEGIN
         IF y0 < y1 THEN
           FOR y := y0 TO y1 DO
             Pixel (x0, y, Farbe)
         ELSE
           FOR y := y0 DOWNTO y1 DO
             Pixel (x0, y, Farbe)
        END
      ELSE
        BEGIN
          dx := x1 - x0;
          dy := y1 - y0;
          m := dy / dx;

          IF (m < 1) AND (m > -1) THEN
            BEGIN
              IF x0 < x1 THEN
                FOR x := x0 TO x1 DO
                  BEGIN
                    y := y0 + ROUND (m * (x-x0));
                    Pixel (x, y, Farbe)
                  END
              ELSE
                FOR x := x0 DOWNTO x1 DO
                  BEGIN
                    y := y0 + ROUND (m * (x-x0));
                    Pixel (x, y, Farbe)
                  END;
            END
          ELSE
            BEGIN
              IF y0 < y1 THEN
                FOR y := y0 TO y1 DO
                  BEGIN
                    x := x0 + ROUND (1/m * (y-y0));
                    Pixel (x, y, Farbe)
                  END
               ELSE
                FOR y := y0 DOWNTO y1 DO
                  BEGIN
                    x := x0 + ROUND (1/m * (y-y0));
                    Pixel (x, y, Farbe)
                  END;
            END;
        END (* IF *)
     END; (* Linie*)

  FUNCTION PixelFarbe (x,y: WORD): BYTE;
   CONST
    ScrSeg = $A000;   (* Segmentadresse des EGA/VGA-Bildspeichers *)

   VAR
    ScrWidth: Word ABSOLUTE $0:$044A;  (* BIOS: Spalten/Bytes pro Zeile *)
    ScrOfs:   Word;
    BitMask, Color:  Byte;

   BEGIN
    y := 479 - y;
    ScrOfs := y * ScrWidth + x DIV 8; (* Bildspeicher-Offsetadresse *)
    BitMask := $80 SHR (x AND 7);    (* Bitmaske berechnen *)
    Color := 0;                      (* Farbe mit 0 initialisieren *)
    Port[$3CE] := 4;                 (* Index Read Map Select Register *)
    Port[$3CF] := 3;                 (* Lesen aus Ebene 3 *)
    IF (Mem[ScrSeg:ScrOfs] AND BitMask) > 0 THEN
     Color := Color + 8;
    Port[$3CF] := 2;                 (* Lesen aus Ebene 2 *)
    IF (Mem[ScrSeg:ScrOfs] AND BitMask) > 0 THEN
     Color := Color + 4;
    Port[$3CF] := 1;                 (* Lesen aus Ebene 1 *)
    IF (Mem[ScrSeg:ScrOfs] AND BitMask) > 0 THEN
     Color := Color + 2;
    Port[$3CF] := 0;                 (* Lesen aus Ebene 0 *)
    IF (Mem[ScrSeg:ScrOfs] AND BitMask) > 0 THEN
     Color := Color + 1;
    PixelFarbe := Color;
  END; (* PixelFarbe *)

  PROCEDURE Schreiben (x, y: WORD; Text: STRING;
                       Vordergrund, Hintergrund: BYTE);
    PROCEDURE  WriteChar(c: Char; x,y: Word; ForeColor, BackColor: Byte);
    (* ------------------------------------------------------ *)
    (* Schreibt in den 16-Farben-Modi das Zeichen c an die    *)
    (* Position x,y (Grafikkoordinaten) mit der Vordergrund-  *)
    (* farbe ForeColor und der Hintergrundfarbe BackColor.    *)
    (* x,y markiert die linke obere Ecke des Zeichens.        *)
    (* ------------------------------------------------------ *)
    CONST
     ScrSeg = $A000;   (* Segmentadresse des EGA/VGA-Bildspeichers *)

    TYPE
     CharMatrix = ARRAY[0..31] OF Byte;   (* Zeichenmatrix-Array *)

    VAR
     CharSize: Word ABSOLUTE $0000:$0485; (* BIOS-Zeichenhhe *)
     FPtr: Pointer ABSOLUTE  $0000:$010C; (* INT 43h-Vektor *)
     CharRow, BitPos, Dummy: Byte;
     FontSeg, FontOfs, ScrOfs, i: Word;
     ScrWidth: Word ABSOLUTE $0:$044A;  (* BIOS: Spalten/Bytes pro Zeile *)

    BEGIN
     y := 479 - y - 16;
     ScrOfs := y * ScrWidth + x DIV 8;  (* Bildspeicher-Offsetadresse *)
     BitPos := x AND 7;                 (* Bitposition berechnen *)
     FontSeg := Seg(FPtr^);             (* Font-Segmentadresse *)
     FontOfs := Ofs(FPtr^) + CharSize * Byte(c); (* Font-Offsetadr. *)
     Port[$3CE] := 5;                   (* Index fr Mode Register *)
     Port[$3CF] := 2;                   (* Schreibmodus 2 einstellen *)
     Port[$3CE] := 8;                   (* Index fr Bit Mask Reg. *)
     FOR i := 0 TO CharSize-1 DO
      BEGIN
       CharRow := Mem[FontSeg:FontOfs];        (* Pixelreihe holen *)
       (* --- linke Seite ausgeben --- *)
       Port[$3CF] := CharRow SHR BitPos;       (* Bitmaske links *)
       Dummy := Mem[ScrSeg:ScrOfs];            (* Lesezyklus erzeugen *)
       Mem[ScrSeg:ScrOfs] := ForeColor;        (* Vordergrundfarbe *)
       Port[$3CF] := (NOT CharRow) SHR BitPos; (* inverse Maske links *)
       Dummy := Mem[ScrSeg:ScrOfs];            (* Lesezyklus erzeugen *)
       Mem[ScrSeg:ScrOfs] := BackColor;        (* Hintergrundfarbe *)
       IF BitPos > 0 THEN
        BEGIN
         (* --- rechte Seite ausgeben --- *)
         Inc(ScrOfs);                          (* Offsetadresse + 1 *)
         Port[$3CF] := CharRow SHL (8-BitPos); (* Bitmaske rechts *)
         Dummy := Mem[ScrSeg:ScrOfs];          (* Lesezyklus erzeugen *)
         Mem[ScrSeg:ScrOfs] := ForeColor;      (* Vordergrundfarbe *)
         Port[$3CF] := (NOT CharRow) SHL (8-BitPos); (* inverse Maske *)
         Dummy := Mem[ScrSeg:ScrOfs];          (* Lesezyklus erzeugen *)
         Mem[ScrSeg:ScrOfs] := BackColor;      (* Hintergrundfarbe *)
         Dec(ScrOfs);                          (* Offsetadresse - 1 *)
        END;
       ScrOfs := ScrOfs + ScrWidth;        (* Offsetadr. nchste Zeile *)
       Inc(FontOfs);                       (* nchste Pixelreihe *)
      END;
     Port[$3CF] := $FF;                (* Default-Bitmaske *)
     Port[$3CE] := 5;                  (* Index fr Mode Register  *)
     Port[$3CF] := 0;                  (* Mode Reg. mit Default laden *)
    END; (* WriteChar *)

  VAR
   i : INTEGER;
  BEGIN (* Schreiben *)
   FOR i := 1 TO LENGTH (Text) DO
    BEGIN
     WriteChar (Text[i], x, y, Vordergrund, Hintergrund);
     INC (x, 8)
    END;
  END; (*  Schreiben *)

FUNCTION Taste: CHAR;
 BEGIN
  Taste := #00;
  WHILE KeyPressed DO
   Taste := ReadKey;
 END; (* Taste *)

END.Graphics


